import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.HTTPCommon.TSBHTTPMethod;
import SecureBlackbox.HTTPServer.*;
import SecureBlackbox.SSLCommon.*;
import SecureBlackbox.SSLServer.*;


public class ClientThread {
	private TElHTTPSServer m_httpsServ;
	private TcpClient m_tcpClient;
	private Logger m_logger;
	private ServerThread m_owner;
	private String m_basePath;
	private Boolean m_upload;
	private TElMemoryStream m_uploadStream;

	private TElHTTPServerRequestParams m_currentRequest;
	private TElHTTPServerResponseParams m_currentResponse;

	private boolean m_disposed;

	public ClientThread(ServerThread owner, TcpClient tcpClient, TElCustomCertStorage certStorage,
		boolean sslMode, Logger logger, String basePath, boolean authBasic, boolean authDigest)
	{
		m_tcpClient = tcpClient;
		m_logger = logger;
		m_owner = owner;
		m_basePath = basePath;

		m_owner.AddConnection(this);

		m_httpsServ = new TElHTTPSServer();

		m_httpsServ.setSSLMode(sslMode);
		m_httpsServ.setCertStorage(certStorage);
		
		m_httpsServ.setAuthBasic(authBasic);
		m_httpsServ.setAuthDigest(authDigest);
		m_httpsServ.setAuthRealm("SBB Demo");

		m_httpsServ.setOnSend(new TSBSendEvent(onSend));
		m_httpsServ.setOnReceive(new TSBReceiveEvent(onReceive));
		m_httpsServ.setOnData(new TSBDataEvent(onData));
		m_httpsServ.setOnOpenConnection(new TSBOpenConnectionEvent(onOpenConnection));
		m_httpsServ.setOnCloseConnection(new SecureBlackbox.SSLCommon.TSBCloseConnectionEvent(onCloseConnection));
		m_httpsServ.setOnSSLError(new TSBErrorEvent(onError));
		m_httpsServ.setOnRequestHeadersReceived(new TSBHTTPRequestHeadersEvent(onRequestHeadersReceived));
		m_httpsServ.setOnRequestBodyReceived(new TSBHTTPRequestBodyEvent(onRequestBodyReceived));
		m_httpsServ.setOnGetUserPassword(new TSBOnGetUserPasswordEvent(onGetPassword));
		

		m_httpsServ.open();
	}
	
	TSBOnGetUserPasswordEvent.Callback onGetPassword = new TSBOnGetUserPasswordEvent.Callback() {
		@Override
		public void tsbOnGetUserPasswordEventCallback(TObject sender, TElHTTPServerRequestParams reqParams, TSBString Password, TSBBoolean accept) {
            accept.value = "pass".equals(Password.value);
		}
	};
	
	
	TSBDataEvent.Callback onData = new TSBDataEvent.Callback() {
		@Override
		public void tsbDataEventCallback(TObject sender, byte[] data) {
				if (m_upload)
					m_uploadStream.write(data, 0, data.length);
		}
	};
	
	TSBHTTPRequestBodyEvent.Callback onRequestBodyReceived = new TSBHTTPRequestBodyEvent.Callback() {
		@Override
		public void tsbhttpRequestBodyEventCallback(TObject arg0,
				TElHTTPServerRequestParams arg1) {
			ProcessRequest();
		}
	};
	
	TSBHTTPRequestHeadersEvent.Callback onRequestHeadersReceived = new TSBHTTPRequestHeadersEvent.Callback() {	
		public void tsbhttpRequestHeadersEventCallback(TObject sender,
				TElHTTPServerRequestParams Request, TSBBoolean accept) {
			String method = "";

			accept.value = true;

			if (m_httpsServ.getAuthBasic() || m_httpsServ.getAuthDigest()){
				Request.setRequireAuth(true);
			}
			
			TSBHTTPMethod m = Request.getMethod();
			
			if (m == TSBHTTPMethod.hmConnect)
				method = "CONNECT";
			if (m == TSBHTTPMethod.hmDelete)
				method = "DELETE";
			if (m == TSBHTTPMethod.hmGet)
				method = "GET";
			if (m == TSBHTTPMethod.hmHead)
				method = "HEAD";
			if (m == TSBHTTPMethod.hmOptions)
				method = "OPTIONS";
			if (m == TSBHTTPMethod.hmPost)
				method = "POST";
			if (m == TSBHTTPMethod.hmPut)
				method = "PUT";
			if (m == TSBHTTPMethod.hmTrace)
				method = "TRACE";
			if (m == TSBHTTPMethod.hmCustom)
				method = Request.getCustomMethod().toUpperCase();

			m_logger.AddRequestEvent(method, Request.getURL());

			m_currentRequest = Request;
			m_currentResponse = new TElHTTPServerResponseParams();
			m_currentRequest.setURL(URLExtractPath(m_currentRequest.getURL()));
			
			m_upload = ("/upload".equals(m_currentRequest.getURL()));
			if (m_upload)
				m_uploadStream = new TElMemoryStream();

			if (m_currentRequest.getURL().compareTo("/") == 0)
				m_currentRequest.setURL("/index.html");

			if ((m_currentRequest.getContentLength() <= 0) && ( ! m_currentRequest.getChunked()))
			{
				ProcessRequest();
			}

		}
	};

	TSBErrorEvent.Callback onError = new TSBErrorEvent.Callback() {
		
		public void tsbErrorEventCallback(TObject sender, int code, boolean fatal,
				boolean remote) {
			Logger.AddLogEvent("Error [Code: " + code + ", Fatal: " + fatal + ", Remote: " + remote + "]");
		}
	};
	
	SecureBlackbox.SSLCommon.TSBCloseConnectionEvent.Callback onCloseConnection = new SecureBlackbox.SSLCommon.TSBCloseConnectionEvent.Callback() {
		@Override
		public void tsbCloseConnectionEventCallback(TObject sender, TSBCloseReason reason) {
			Logger.AddLogEvent("Connection closed [" + reason + "]");
			m_tcpClient.close();
		}
	};
	
	TSBOpenConnectionEvent.Callback onOpenConnection = new TSBOpenConnectionEvent.Callback() {
		
		public void tsbOpenConnectionEventCallback(TObject sender) {
			Logger.AddLogEvent("New connection");
		}
	};
	
	TSBReceiveEvent.Callback onReceive = new TSBReceiveEvent.Callback() {
		
		public void tsbReceiveEventCallback(TObject sender, byte[] Buffer, int MaxSize, TSBInteger written) {
			written.value = 0;
			try
			{
				written.value = m_tcpClient.read(Buffer, 0, MaxSize);
				if (written.value < 0)
					written.value = 0;
			}
			catch (Exception ex)
			{
				written.value = 0;
			}
            if (written.value == 0)
				m_tcpClient.close();
			
		}
	};
	
	TSBSendEvent.Callback onSend = new TSBSendEvent.Callback() {
		
		public void tsbSendEventCallback(TObject sender, byte[] Buffer) {
			m_tcpClient.write(Buffer, 0, Buffer.length);
		}
	};
	
	public void Dispose()
	{
		if (! m_disposed)
		{
			m_httpsServ.close(false);
			m_tcpClient.close();

			m_owner.RemoveConnection(this);
			m_disposed = true;
		}
	}

	private String URLExtractPath(String URL)
	{
		if ((URL.length() > 0) && (URL.charAt(0) == '/'))
		{
			return URL;
		}
		else
		{
			TElParseURLResult result = SBStrUtils.parseURL(URL, true);
			
			return result.Path;
		}
	}


    private void saveFiles(TElMultipartFormList mpl)
    {
        TElMultipartFormData mfd;
        for (int i = 0; i < mpl.getCount(); i++)
        {
            mfd = mpl.getValuesByIndex(i);
            if (mfd instanceof TElMultipartFormList)
            {
                saveFiles((TElMultipartFormList)mfd);
            }
            else if (mfd.getFileName() != "")
            {
            	try{
            	FileOutputStream fos = new FileOutputStream(m_basePath + "/" + mfd.getFileName());
				try {
					fos.write(m_uploadStream.getBuffer(), (int)mfd.getContentStartPos(), (int)mfd.getContentSize());
				} finally {
					fos.close();
				}  
            	}catch (Exception ex)
				{
					m_currentResponse.setStatusCode(500);
				}
            }
        }
    }

	
	private void ProcessRequest()
	{
		TElFileStream FS = null;

		if (m_currentRequest.getMethod() == TSBHTTPMethod.hmGet)
		{
			try
			{
				FS = new TElFileStream(m_basePath + m_currentRequest.getURL(), "r", true);
				m_currentResponse.setContentLength(FS.getLength());
				m_currentResponse.setStatusCode(200);
			}
			catch (Exception ex)
			{
				m_currentResponse.setStatusCode(404);
			}
		}
		else if ((m_currentRequest.getMethod() == TSBHTTPMethod.hmPost) && m_upload) 
		{
				m_uploadStream.setPosition(0);
				TElMultipartFormList mpl = m_httpsServ.parseMultipartFormData(m_currentRequest, m_uploadStream, false);
				saveFiles(mpl);
				m_currentResponse.setStatusCode(301);
				m_currentResponse.setLocation("upload.html");
		} 
		else
		{
			m_currentResponse.setStatusCode(400);
		}

		m_httpsServ.sendResponseHeaders(m_currentRequest, m_currentResponse);
		if (FS != null)
		{
			m_httpsServ.sendResponseData(FS);
			FS.Free();
		}
		m_httpsServ.responseComplete();
  
		if ( ! m_currentRequest.getKeepAlive())
			Dispose();
		}

		public void Execute()
		{			
			while (true)
			{
				try
				{
					if (m_tcpClient.GetDataAvailable())
					{
						m_httpsServ.dataAvailable();
					}
					else
					{
						java.lang.Thread.sleep(50);
					}
				}
				catch (Exception ex)
				{
					break;
				} 
			}

			Dispose();
	}

}
