import java.io.*;
import java.util.*;

import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.SSHCommon.*;

public class DemoSettings {
	
	// user information file
	private String m_UsersFile = "";

	// server private key file
	private String m_ServerKeyFile = "";
	
	// network settings file
	private String m_IPFile = "";
	
	// server configuration
	private boolean m_Empty = true;
	private ArrayList<UserInfo> m_Users = new ArrayList<UserInfo>();
	private String m_ServerKey = "";
	private String m_ServerHost = "127.0.0.1";
	private int m_ServerPort = 22;
	private boolean m_ForceCompression = false;
	
	/**
	 * Indicates if demo settings are loaded
	 */
	public boolean getIsEmpty() {
		return m_Empty;
	}
	
	/**
	 * Stores information about registered users
	 */
	public ArrayList<UserInfo> getUsers(){
		return m_Users;
	}
	
	/**
	 * Get private key of the host
	 */
	public String getServerKey() {
		return m_ServerKey;
	}
	
	/**
	 * Set private key of the host
	 */
	public void setServerKey(String value) {
		m_ServerKey = value;
	}
	
	/**
	 * Get server socket address
	 */
	public String getServerHost() {
		return m_ServerHost;
	}
	
	/**
	 * Set server socket address
	 */
	public void setServerHost(String value) {
		m_ServerHost = value;
	}

	/*
	 * Set server port
	 */
	public void setServerPort(int value) {
		m_ServerPort = value;
	}
	
	/*
	 * Get server port
	 */
	public int getServerPort() {
		return m_ServerPort;
	}
	
	/*
	 * Set compression flag
	 */
	public void setForceCompression(boolean value) {
		m_ForceCompression = value;
	}
	
	/*
	 * Get compression flag
	 */
	public boolean getForceCompression() {
		return m_ForceCompression;
	}

	public boolean loadSettings() {
		try
		{
			String AppPath = new java.io.File("").getAbsolutePath();;
			AppPath = AppPath + "\\SecureBlackbox\\";
			File appDir = new File(AppPath);
			if (!appDir.exists()) appDir.mkdir();
			m_UsersFile = AppPath + "sshsrvusers";
			m_ServerKeyFile = AppPath + "SSHSrvHostkey";
			m_IPFile = AppPath + "sshbindings";
			if ((!new File(m_UsersFile).exists()) || (!new File(m_ServerKeyFile).exists()))
				m_Empty = true;
			else
				m_Empty = false;
			// Read user information
			FileInputStream fstream = new FileInputStream(m_UsersFile);
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
	
			String line = null;
			while ((line = br.readLine()) != null) {
				if (line.length() > 0) 
				{
					UserInfo ui = new UserInfo(line);
					m_Users.add(ui);
				}
			}
			
			br.close();
			in.close();
			fstream.close();
			
			// Read server public key
			try
			{
				String AServerKey = readFileAsString(m_ServerKeyFile);
				TElSSHKey Key = new TElSSHKey();
				int Result = Key.loadPrivateKey(m_ServerKeyFile, "");
				if (Result == 0) 
					m_ServerKey = AServerKey;
			}
			catch(Exception exc) 
			{
				Logger.Log("Unable to load server key " + exc.getMessage(), true);
				return false;
			}
			// Read IP address
			try
			{
				String AServerHost = readFileAsString(m_IPFile);
				String[] par = AServerHost.split(":");
				m_ServerHost = par[0];
				m_ServerPort = Integer.parseInt(par[1]);
			}
			catch(Exception exc) 
			{
				Logger.Log("Unable to read host:posrt " + exc.getMessage(), true);
				m_ServerHost = "127.0.0.1";
				m_ServerPort = 22;
			}
			return !getIsEmpty();
		}
		catch(Exception exc) 
		{
			Logger.Log("Error reading settings " + exc.getMessage(), true);
			return false;
		}
	}

	public void SaveSettings()
	{
		try
		{
			// Users
			BufferedWriter sw = new BufferedWriter(new FileWriter(m_UsersFile));
			for (int i = 0; i < m_Users.size(); i++)
				sw.write(m_Users.get(i).ToString());

			sw.close();
			
			// Server private key
			sw = new BufferedWriter(new FileWriter(m_ServerKeyFile));			
			sw.write(getServerKey());
			sw.close();
			
			// Server IP bindings
			sw = new BufferedWriter(new FileWriter(m_IPFile));			
			sw.write(getServerHost() + ":" + getServerPort());
			sw.close();
		} 
		catch(Exception exc) 
		{
			Logger.Log("SaveSettings : " + exc.getMessage(),true);
		}
	}

	private static String readFileAsString(String filePath) throws java.io.IOException{
	    byte[] buffer = new byte[(int) new File(filePath).length()];
	    BufferedInputStream f = null;
	    try {
	        f = new BufferedInputStream(new FileInputStream(filePath));
	        f.read(buffer);
	    } finally {
	        if (f != null) try { f.close(); } catch (IOException ignored) { }
	    }
	    return new String(buffer);
	}
	
	public UserInfo FindUser(String UserName)
	{
		UserInfo result = null;
		if (UserName == null) return null;
		for (int i = 0; i < m_Users.size(); i++) 
		{
			if (UserName.compareTo(((UserInfo)m_Users.get(i)).getName()) == 0) 
				result = (UserInfo)m_Users.get(i);
		}
		return result;
	}
	
	public boolean AddUser()
	{
		m_Users.add(new UserInfo());
		return true;
	}
	
	public boolean RemoveUser(int UserNumber)
	{
		try
		{
			m_Users.remove(UserNumber);
			return true;
		}
		catch(Exception ex) 
		{
			return(false);
		}
	}	
	
	public boolean EditUser(int Number)
	{
		if (Number < 0) 
		{
			return false;
		}
		UserInfo info = (UserInfo)m_Users.get(Number);
		boolean Result = UserDialog.EditUserParameters(info);
		if (Result) 
		{
			SaveSettings();
			return true;
		}
		return false;
	}	
		
	public boolean SetHostKey(String HostKey)
	{
		try
		{
			TElSSHKey Key = new TElSSHKey();
			int Result = Key.loadPrivateKey(SBUtils.bytesOfString(HostKey), HostKey.length(),"");
			return Result == 0;
		} 
		catch(Exception exc) 
		{
			Logger.Log("SetHostKey : " + exc.getMessage(),true);
		}
		return false;
	}	
}
